package cn.com.ctop.okr.controller;

import cn.com.ctop.okr.constant.CommonConstant;
import cn.com.ctop.okr.entity.QYWxUserInfo;
import cn.com.ctop.okr.qywx.constant.QYWxConstant;
import cn.com.ctop.okr.service.QYWxService;
import cn.com.ctop.okr.shiro.vo.DefContants;
import cn.com.ctop.okr.utils.JwtUtil;
import cn.com.ctop.okr.utils.RedisUtil;
import cn.com.ctop.okr.utils.oConvertUtils;
import cn.com.ctop.okr.vo.LoginUser;
import cn.com.ctop.okr.vo.Result;
import com.alibaba.fastjson.JSONObject;
import lombok.val;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.api.WxCpOAuth2Service;
import me.chanjar.weixin.cp.api.impl.WxCpServiceImpl;
import me.chanjar.weixin.cp.bean.WxCpOauth2UserInfo;
import me.chanjar.weixin.cp.config.impl.WxCpDefaultConfigImpl;
import org.apache.log4j.Logger;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@RestController
@RequestMapping("/sys")
public class LoginController {
    @Autowired
    private QYWxService qyWxService;
    @Autowired
    private RedisUtil redisUtil;

    @RequestMapping(value = "/wxlogin", method = RequestMethod.POST)
    public Result<JSONObject> wxlogin(@RequestParam(name = "code") String code) throws Exception {
        Result<JSONObject> result = new Result<>();
        val configStorage = new WxCpDefaultConfigImpl();
        configStorage.setCorpId(QYWxConstant.QYWX_CORP_ID);
        configStorage.setAgentId(QYWxConstant.QYWX_AGENT_ID);
        configStorage.setCorpSecret(QYWxConstant.QYWX_CORP_SECRET);
        configStorage.setToken("hcst2019");
        configStorage.setAesKey("hcst2019");

        //configStorage.setCorpId("ww24b8a47826f5875f");
        //configStorage.setAgentId(1000002);
        //configStorage.setCorpSecret("MJIlySADGPlgvADnkFQPUpfZD4yV-4WN6066OgNnb0s");
        //configStorage.setToken("hcst2019");
        //configStorage.setAesKey("hcst2019");
        val service = new WxCpServiceImpl();

        service.setWxCpConfigStorage(configStorage);
        WxCpOAuth2Service wxCpOAuth2Service = service.getOauth2Service();
        try {
            WxCpOauth2UserInfo userInfo = wxCpOAuth2Service.getUserInfo(QYWxConstant.QYWX_AGENT_ID, code);
            if (userInfo != null && userInfo.getUserId() != null) {
                //WxCpUserService wxCpUserService = new WxCpUserServiceImpl(service);
                //WxCpUser user = wxCpUserService.getById(userInfo.getUserId());
                QYWxUserInfo user = qyWxService.getUserInfoByUserid(userInfo.getUserId());
                if(user.getStatus() != 1){
                    result.error500("该用户处于非正常状态！");
                    return result;
                }

                //用户登录信息
                Result<JSONObject> getResult = userInfo(user, result);
                return getResult;

            }
        } catch (WxErrorException e) {
            result.error500("企业微信授权错误");
            e.printStackTrace();
        }
        return result;
    }

    /**
     * 用户信息
     *
     * @param user
     * @param result
     * @return
     */
    private Result<JSONObject> userInfo(QYWxUserInfo user, Result<JSONObject> result) {
        String openId = user.getOpenId();  //openId
        String userid = user.getUserid();  //userId
        // 生成token
        String token = JwtUtil.sign(openId, userid);
        redisUtil.set(CommonConstant.PREFIX_USER_TOKEN + token, token);
        //设置超时时间
        redisUtil.expire(CommonConstant.PREFIX_USER_TOKEN + token, JwtUtil.EXPIRE_TIME / 1000);

        // 获取用户部门信息
        JSONObject obj = new JSONObject();
        //List<CorpDepartInfo> departs = corpUserInfoService.queryUserDeparts(user.getOpenId());
        //obj.put("departs", departs);
        //if (departs == null || departs.size() == 0) {
        //    obj.put("multi_depart", 0);
        //} else if (departs.size() == 1) {
        //    //sysUserService.updateUserDepart(username, departs.get(0).getOrgCode());
        //    obj.put("multi_depart", 1);
        //} else {
        //    obj.put("multi_depart", 2);
        //}
        obj.put("token", token);
        obj.put("userInfo", user);
        result.setResult(obj);
        result.success("登录成功");
        return result;
    }

    /**
     * 退出登录
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/logout")
    public Result<Object> logout(HttpServletRequest request, HttpServletResponse response) {
        //用户退出逻辑
        //Subject subject = SecurityUtils.getSubject();
        //QYWxUserInfo sysUser = (QYWxUserInfo) subject.getPrincipal();

        //logger.info(" 用户名:  " + sysUser.getName() + ",退出成功！ ");
        //subject.logout();

        //if(){
        //
        //}HttpServletRequest httpServletRequest = (HttpServletRequest) request;

        String token = request.getHeader(DefContants.X_ACCESS_TOKEN);
        //String openId = JwtUtil.getUsername("");
        //String cacheToken = String.valueOf(redisUtil.get(CommonConstant.PREFIX_USER_TOKEN + token));

        if (token != null) {
            redisUtil.del(CommonConstant.PREFIX_USER_TOKEN + token);
        }
        return Result.ok("退出登录成功！");
    }


}
