package cn.com.ctop.okr.service.impl;

import cn.com.ctop.okr.entity.KrAlignInfo;
import cn.com.ctop.okr.entity.OkrKrInfo;
import cn.com.ctop.okr.entity.QYWxUserInfo;
import cn.com.ctop.okr.mapper.KrAlignInfoMapper;
import cn.com.ctop.okr.service.IKrAlignInfoService;
import cn.com.ctop.okr.service.IQYWxUserInfoService;
import cn.com.ctop.okr.utils.ResultMapUtils;
import cn.com.ctop.okr.utils.StatusCode;
import cn.com.ctop.okr.vo.Result;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class KrAlignInfoServiceImpl extends ServiceImpl<KrAlignInfoMapper, KrAlignInfo> implements IKrAlignInfoService {
    @Autowired
    private KrAlignInfoMapper krAlignInfoMapper;
    @Autowired
    private IQYWxUserInfoService qyWxUserInfoService;

    @Override
    public Map<String, Object> addAlign(Long originId, String alignIds) {
        Map<String,Object>result = new HashMap<>();
        if(null == originId||null == alignIds||alignIds.trim().equals("")){
            ResultMapUtils.setResultMap(result,StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        String[]alignIdArray = alignIds.split(",");
        for(int i=0;i<alignIdArray.length;i++){
            Long alignId = Long.parseLong(alignIdArray[i]);
            QueryWrapper<KrAlignInfo>queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("origin_kr",originId);
            queryWrapper.eq("align_kr",alignId);
            queryWrapper.last("limit 1");
            KrAlignInfo alignInfo = this.getOne(queryWrapper);
            if(null!=alignInfo){
                continue;
            }
            alignInfo = new KrAlignInfo();
            alignInfo.setAlignKr(alignId);
            alignInfo.setOriginKr(originId);
            alignInfo.setStatus(1);
            alignInfo.setCreateTime(new Date());
            alignInfo.setUpdateTime(new Date());
            this.saveOrUpdate(alignInfo);
        }
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }

    @Override
    public Map<String, Object> deleteAlign(Long originId, Long alignId) {
        Map<String,Object>result = new HashMap<>();
        QueryWrapper<KrAlignInfo>queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("origin_kr",originId);
        queryWrapper.eq("align_kr",alignId);
        queryWrapper.last("limit 1");
        KrAlignInfo alignInfo = this.getOne(queryWrapper);
        if(null==alignInfo){
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
            return result;
        }
        this.removeById(alignInfo.getId());
        ResultMapUtils.setResultMap(result, StatusCode.COMMON_SUCCESS);
        return result;
    }
    @Override
    public Map<String, Object> alignListByParams(Long originId) {
        Map<String,Object>result = new HashMap<>();
        if(null == originId){
            ResultMapUtils.setResultMap(result,StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        List<OkrKrInfo> myAligns = krAlignInfoMapper.selectMyAlign(originId);
        if(null!=myAligns&&!myAligns.isEmpty()){
            myAligns.forEach(detail->{
                QYWxUserInfo userInfo = qyWxUserInfoService.getByOpenId(detail.getUserId());
                if(null!=userInfo){
                    detail.setUserName(userInfo.getName());
                }
            });
        }
        List<OkrKrInfo> otherAligneds = krAlignInfoMapper.selectOtherAligned(originId);
        if(null!=otherAligneds&&!otherAligneds.isEmpty()){
            otherAligneds.forEach(detail->{
                QYWxUserInfo userInfo = qyWxUserInfoService.getByOpenId(detail.getUserId());
                if(null!=userInfo){
                    detail.setUserName(userInfo.getName());
                }
            });
        }
        ResultMapUtils.setResultMap(result,StatusCode.COMMON_SUCCESS);
        result.put("myAlign",myAligns);
        result.put("otherAligned",otherAligneds);
        return result;
    }
}
