package cn.com.ctop.okr.service.impl;

import cn.com.ctop.okr.entity.OkrFocusInfo;
import cn.com.ctop.okr.entity.OkrObjectInfo;
import cn.com.ctop.okr.entity.QYWxUserInfo;
import cn.com.ctop.okr.mapper.OkrFocusInfoMapper;
import cn.com.ctop.okr.service.IOkrFocusInfoService;
import cn.com.ctop.okr.service.IOkrObjectInfoService;
import cn.com.ctop.okr.service.IQYWxUserInfoService;
import cn.com.ctop.okr.utils.ResultMapUtils;
import cn.com.ctop.okr.utils.StatusCode;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.util.HashMap;
import java.util.Map;

/**
 * 关注信息
 * @author jeecg-boot
 * @date   2020-09-05
 * @version V1.0
 */
@Service
public class OkrFocusInfoServiceImpl extends ServiceImpl<OkrFocusInfoMapper, OkrFocusInfo> implements IOkrFocusInfoService {
    @Autowired
    private IQYWxUserInfoService qyWxUserInfoService;
    @Autowired
    private OkrFocusInfoMapper okrFocusInfoMapper;

    @Override
    public void removeByParams(String followId, String userId) {
        Map<String,Object>params = new HashMap<>();
        if(null!=followId&&!"".equals(followId.trim())){
            params.put("follow_id",followId);
        }
        if(null!=userId&&!userId.trim().equals("")){
            params.put("user_id",userId);
        }
        okrFocusInfoMapper.deleteByMap(params);
    }

    @Override
    public OkrFocusInfo getByParams(String followId, String userId) {
        QueryWrapper<OkrFocusInfo>queryWrapper = new QueryWrapper<>();
        if(null!=followId&&!"".equals(followId.trim())){
            queryWrapper.eq("follow_id",followId);
        }
        if(null!=userId&&!userId.trim().equals("")){
            queryWrapper.eq("user_id",userId);
        }
        queryWrapper.last("limit 1");
        return this.getOne(queryWrapper);
    }

    @Override
    public Map<String, Object> focus(JSONObject data) {
        Map<String,Object> result = new HashMap<>();
        String followId = data.getString("userId");
        Integer status = data.getInteger("status");
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        if(followId == null||"".equals(followId.trim())||status == null){
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        QYWxUserInfo qyWxUserInfo = qyWxUserInfoService.getByOpenId(userId);
        if(null == qyWxUserInfo){
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }
        if(status == 0){
            this.removeByParams(followId,userId);
        }else {
            OkrFocusInfo focusInfo = new OkrFocusInfo();
            focusInfo.setFollowId(followId);
            focusInfo.setUserId(userId);
            this.save(focusInfo);
        }
        ResultMapUtils.setResultMap(result,StatusCode.COMMON_SUCCESS);
        return result;
    }
}
