package cn.com.ctop.okr.service.impl;

import cn.com.ctop.okr.controller.QyWxController;
import cn.com.ctop.okr.dto.UserInfoDto;
import cn.com.ctop.okr.entity.*;
import cn.com.ctop.okr.mapper.*;
import cn.com.ctop.okr.service.*;
import cn.com.ctop.okr.utils.*;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
public class OkrServiceImpl implements OkrService {
    @Resource
    private OkrContentStatusMapper okrContentStatusMapper;
    @Resource
    private OkrKrInfoMapper okrKrInfoMapper;
    @Resource
    private OkrMemberMapper okrMemberMapper;
    @Autowired
    private IOkrObjectInfoService okrObjectInfoService;
    @Autowired
    private IOkrTaskInfoService okrTaskInfoService;
    @Autowired
    private IQYWxUserInfoService wxUserInfoService;
    @Autowired
    private IOkrTaskInfoDisclosureService okrTaskInfoDisclosureService;
    @Autowired
    private IOkrTaskInfoRemindService okrTaskInfoRemindService;
    @Autowired
    private IOkrObjectInfoService objectInfoService;
    @Autowired
    IOkrLogService okrLogService;


    @Override
    public Map<String, Object> getOkrInfo(UserInfoDto data){
        Map<String, Object> result = new HashMap<>();
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String currentUserId = sysUser.getOpenId();
        Long objectId = data.getObjectId();
        OkrObjectInfo objectInfo =objectInfoService.getById(objectId);
        if(null == objectInfo){
            ResultMapUtils.setResultMap(result, StatusCode.COMMON_PARAM_ERROR);
            return result;
        }

        //本周任务信息
        List<OkrTaskInfo> taskInfoList = okrTaskInfoService.getTaskInfo(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter(), 1);
        List<OkrTaskInfo>setTaskList =checkTaskInfo(taskInfoList,currentUserId);

        //未来四周任务信息
        List<OkrTaskInfo> taskFourWeekList = okrTaskInfoService.getTaskInfo(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter(), 2);
        List<OkrTaskInfo>setTaskFourWeekList =checkTaskInfo(taskFourWeekList,currentUserId);

        //OKR信息
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoService.getQuarterObject(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());
        List<Map<String, Object>> okrList = new ArrayList<>();
        for (OkrObjectInfo okrObjectInfo:OkrObjectInfoList){
            Map<String, Object> okrMap = new HashMap<>();
            List<OkrKrInfo> krList = okrKrInfoMapper.getKrByObjectId(okrObjectInfo.getId(),objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());
            List<OkrKrInfo>setKrList =checkKrList(krList,currentUserId);
            okrMap.put("objectInfo",okrObjectInfo);
            okrMap.put("krList",setKrList);
            okrList.add(okrMap);
        }

        //状态信息
        List<OkrContentStatus> contentStatusList = okrContentStatusMapper.getContentStatusList(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());
        String now = DateUtils.formatDate();
        //当前日期数据
        OkrWeekData weekData = weekDataService.getWeekByDate(now);
        List<Integer> months = DateUtils.getMonthsByQuarter(objectInfo.getQuarter());
        //根据月份数据获取周
        Map<Integer, Object> weeks = weekDataService.getWeeksByMonths(objectInfo.getYear(),months);
        Map<String,Object> timeMap = new HashMap<>();
        timeMap.put("year",objectInfo.getYear());
        timeMap.put("quarter",objectInfo.getQuarter());
        Integer month = DateUtils.getMonth();
        Integer week = DateUtils.getWeekByDate(new Date());
        timeMap.put("currentMonth",month);
        timeMap.put("currentWeek",week);
        timeMap.put("months",months);
        timeMap.put("weeks",weeks);
        result.put("time", timeMap);
        result.put("taskInfoList",transformTaskInfo(setTaskList, 1));
        result.put("taskFourWeekList",transformTaskInfo(setTaskFourWeekList, 2));
        result.put("okr",okrList);
        result.put("status", contentStatusList);
        return result;
    }

    private List<OkrTaskInfo> checkTaskInfo(List<OkrTaskInfo> taskInfoList, String openId) {
        List<OkrTaskInfo>setList = new ArrayList<>();
        if(null!=taskInfoList&&!taskInfoList.isEmpty()){
            setList = new ArrayList<>();
            for(OkrTaskInfo taskInfo:taskInfoList){
                //1:查询当前kr使用可以被展示
                OkrTaskInfo getTaskInfo = okrTaskInfoService.checkShow(taskInfo,openId);
                if(null!=getTaskInfo){
                    setList.add(getTaskInfo);
                }
            }
        }
        return setList;
    }


    @Override
    public void insertOkrMember(List<String> memberList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        okrMemberMapper.deleteOkrMemberByUserId(userId,year,quarter);
        okrMemberMapper.insertOkrMember(userId, memberList, year, quarter);
    }

    @Override
    public void insertOkrMember2(List<String> memberList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        okrMemberMapper.insertOkrMember(userId, memberList, year, quarter);
    }

    @Override
    public void deleteOkrMember(String memberId){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        okrMemberMapper.deleteOkrMemberByUserId2(userId, memberId, year, quarter);
    }
    @Override
    public List<UserInfoDto> getMemberListByUserId(String name){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        return okrMemberMapper.getMemberListByUserId(userId, name, year, quarter);
    }
    //保存任务

    @Override
    public void insertTaskInfo(OkrTaskInfo taskInfo){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        String now = DateUtils.formatDate();
        OkrWeekData weekData = null;
        if(null == taskInfo.getWeek()||null == taskInfo.getYear()){
            weekData = weekDataService.getWeekByDate(now);
        }else{
            weekData = weekDataService.getWeekByYearAndWeek(taskInfo.getYear(),taskInfo.getWeek());
        }
        taskInfo.setUserId(userId);
        taskInfo.setYear(weekData.getYear());
        taskInfo.setQuarter(weekData.getQuarter());
        taskInfo.setWeek(weekData.getWeek());
        taskInfo.setMonth(weekData.getMonth());
        okrTaskInfoService.insertTaskInfo(taskInfo);

        insertTaskPublicIds(taskInfo,taskInfo.getPublicIds());
        insertTaskRemindIds(sysUser.getName(),taskInfo,taskInfo.getRemindIds());
        //日志留存
        Integer opration;
        if(taskInfo.getType() == 1){
            opration = 3;
        }else{
            opration = 4;
        }
        okrLogService.insertOkrLog(1,opration,taskInfo.getId(),weekData.getYear(),weekData.getQuarter(),JSONObject.toJSONString(taskInfo),userId);
    }
    private void insertTaskPublicIds(OkrTaskInfo taskInfo,List<String>publicIds){
        if(null!=publicIds&&!publicIds.isEmpty()){
            for(String publicId:publicIds){
                OkrTaskInfoDisclosure disclosure = new OkrTaskInfoDisclosure();
                disclosure.setExtraId(publicId);
                String[] ids = publicId.split("##");
                disclosure.setOkrTaskId(taskInfo.getId());
                //部门
                if(ids.length == 1){
                    disclosure.setPublicType(1);
                }
                if(ids.length == 2){
                    disclosure.setPublicType(2);
                }
                disclosure.setPublicId(ids[0]);
                disclosure.setStatus(1);
                okrTaskInfoDisclosureService.save(disclosure);
            }
        }
    }
    private void insertTaskRemindIds(String userName,OkrTaskInfo taskInfo,List<String>remindIds){
        if(null!=remindIds&&!remindIds.isEmpty()){
            for(String remindId:remindIds){
                OkrTaskInfoRemind remind = new OkrTaskInfoRemind();
                remind.setExtraId(remindId);
                String[] ids = remindId.split("##");
                remind.setOkrTaskId(taskInfo.getId());
                //部门
                if(ids.length == 1){
                    remind.setRemindType(1);
                    Long departId = Long.parseLong(ids[0]);
                    List<String>wechatIds;
                    //@所有人
                    if(null!=departId&&departId == 10086){
                        wechatIds = qyWxDepartInfoService.getAllUserId();
                    }else {
                        wechatIds = qyWxDepartInfoService.getUserIdByDepartId(Long.parseLong(ids[0]));
                    }
                    CorpWexinUtils.sendMessage(wechatIds,userName+"在okr中@了你: "+taskInfo.getTaskTitle());
                }
                if(ids.length == 2){
                    remind.setRemindType(2);
                    QYWxUserInfo userInfo = wxUserInfoService.getByOpenId(ids[0]);
                    if(null!=userInfo){
                        CorpWexinUtils.sendMessageByWeChatId(userInfo.getUserid(),userName+"在okr中@了你: "+taskInfo.getTaskTitle());
                    }
                }
                remind.setRemindId(ids[0]);
                remind.setStatus(1);
                okrTaskInfoRemindService.save(remind);

            }
        }
    }

    @Autowired
    private QYWxDepartInfoService qyWxDepartInfoService;
    //更新任务
    @Override
    public void updateTaskInfo(OkrTaskInfo taskInfo){
        //1:查询taskInfo
        OkrTaskInfo getTask = okrTaskInfoService.getById(taskInfo.getId());
        if(null!=taskInfo.getTaskTitle()&&!"".equals(taskInfo.getTaskTitle())){
            getTask.setTaskTitle(taskInfo.getTaskTitle());
        }
        if(null!=taskInfo.getTaskInfo()&&!"".equals(taskInfo.getTaskInfo())){
            getTask.setTaskInfo(taskInfo.getTaskInfo());
        }
        if(null!=taskInfo.getPermission()){
            getTask.setPermission(taskInfo.getPermission());
        }
        if(null!=taskInfo.getPriority()&&!"".equals(taskInfo.getPriority())){
            getTask.setPriority(taskInfo.getPriority());
        }
        if(null!=taskInfo.getStatus()){
            getTask.setStatus(taskInfo.getStatus());
        }
        okrTaskInfoService.updateById(getTask);
        if(taskInfo.getStatus() == 2){
            if(getTask.getType() == 2){
                //月份信息数据
                getTask.setStatus(1);
                int month = getTask.getMonth()+1;
                int quarter = DateUtils.getQuarterByMonthNum(month);
                if(month == 12){
                    getTask.setMonth(month);
                }else{
                    getTask.setMonth(month%12);
                }
                getTask.setQuarter(quarter);
                getTask.setId(null);
                okrTaskInfoService.insertTaskInfo(getTask);
            }else{
                OkrWeekData getNextWeek = weekDataService.getWeekByYearAndWeek(getTask.getYear(),getTask.getWeek()+1);
                getTask.setStatus(1);
                getTask.setWeek(getNextWeek.getWeek());
                getTask.setMonth(getNextWeek.getMonth());
                getTask.setYear(getNextWeek.getYear());
                getTask.setId(null);
                okrTaskInfoService.insertTaskInfo(getTask);
            }
        }
        /**
         * 需要先删除历史数据
         */
        okrTaskInfoDisclosureService.deleteByTaskId(taskInfo.getId());
        okrTaskInfoRemindService.deleteByTaskId(taskInfo.getId());
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        getTask.setStatus(taskInfo.getStatus());
        getTask.setId(taskInfo.getId());
        List<String>publicIds = taskInfo.getPublicIds();
        List<String>remindIds = taskInfo.getRemindIds();
        insertTaskPublicIds(getTask,publicIds);
        insertTaskRemindIds(sysUser.getName(),getTask,remindIds);
    }
    //保存任务

    @Override
    public void insertContentStatus(OkrContentStatus content){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        content.setUserId(userId);
        content.setYear(DateUtils.getYear(new Date()));
        content.setQuarter(DateUtils.getQuarter(new Date()));
        okrContentStatusMapper.insertContentStatus(content);

        //日志留存
        okrLogService.insertOkrLog(1,5,content.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(content),userId);
    }
    //更新任务

    @Override
    public void updateContentStatus(OkrContentStatus content){
        okrContentStatusMapper.updateContentStatus(content);
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        okrLogService.insertOkrLog(2,5,content.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(content),userId);
    }
    //同时保存O和krList

    @Override
    public void insertObjectAndKrList(OkrObjectInfo objectInfo, List<OkrKrInfo> krList){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        okrObjectInfoService.insertObjectInfo(objectInfo,userId,year,quarter);
        //日志留存
        okrLogService.insertOkrLog(1,1,objectInfo.getId(),year,quarter,JSONObject.toJSONString(objectInfo),userId);

        for(OkrKrInfo krInfo:krList){
            krInfo.setUserId(userId);
            krInfo.setObjectId(objectInfo.getId());
            krInfo.setYear(year);
            krInfo.setQuarter(quarter);
            okrKrInfoMapper.insertOkrKrInfo(krInfo);
            insertKrPublicIds(krInfo,krInfo.getPublicIds());
            insertKrRemindIds(sysUser.getName(),krInfo,krInfo.getRemindIds());
            //日志留存
            okrLogService.insertOkrLog(1,2,krInfo.getId(),year,quarter,JSONObject.toJSONString(krInfo),userId);
        }
    }

    private void insertKrRemindIds(String username,OkrKrInfo krInfo,List<String>remindIds){
        if(null!=remindIds&&!remindIds.isEmpty()){
            for(String remindId:remindIds){
                OkrObjectInfoRemind remind = new OkrObjectInfoRemind();
                remind.setExtraId(remindId);
                String[] ids = remindId.split("##");
                remind.setKrId(krInfo.getId());
                //部门
                if(ids.length == 1){
                    remind.setRemindType(1);
                    Long departId = Long.parseLong(ids[0]);
                    List<String>wechatIds;
                    //@所有人
                    if(null!=departId&&departId == 10086){
                        wechatIds = qyWxDepartInfoService.getAllUserId();
                    }else {
                        wechatIds = qyWxDepartInfoService.getUserIdByDepartId(Long.parseLong(ids[0]));
                    }
                    if(null!=wechatIds&&!wechatIds.isEmpty()){
                        CorpWexinUtils.sendMessage(wechatIds,username+"在OKR中@你了: "+krInfo.getKrInfo());
                    }
                }
                if(ids.length == 2){
                    remind.setRemindType(2);
                    QYWxUserInfo userInfo = wxUserInfoService.getByOpenId(ids[0]);
                    if(null!=userInfo){
                        CorpWexinUtils.sendMessageByWeChatId(userInfo.getUserid(),username+"在OKR中@你了: "+krInfo.getKrInfo());
                    }
                }
                remind.setRemindId(ids[0]);
                remind.setStatus(1);
                okrObjectInfoRemindService.save(remind);
            }
        }
    }
    private void insertKrPublicIds(OkrKrInfo krInfo,List<String>publicIds){
        if(null!=publicIds&&!publicIds.isEmpty()){
            for(String publicId:publicIds){
                OkrObjectInfoDisclosure disclosure = new OkrObjectInfoDisclosure();
                disclosure.setExtraId(publicId);
                String[] ids = publicId.split("##");
                disclosure.setKrId(krInfo.getId());
                //部门
                if(ids.length == 1){
                    disclosure.setPublicType(1);
                }
                if(ids.length == 2){
                    disclosure.setPublicType(2);
                }
                disclosure.setPublicId(ids[0]);
                disclosure.setStatus(1);
                okrObjectInfoDisclosureService.save(disclosure);
            }
        }
    }
    @Override
    public void updateObject(OkrObjectInfo objectInfo){
        okrObjectInfoService.updateObjectInfo(objectInfo);
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        okrLogService.insertOkrLog(2,1,objectInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(objectInfo),userId);
    }
    @Autowired
    private IKrInfoservice krInfoservice;

    @Override
    public void updateKr(OkrKrInfo krInfo){
        List<String>publicIds = krInfo.getPublicIds();
        List<String>remindIds = krInfo.getRemindIds();
        krInfoservice.updateKrInfo(krInfo);
        okrObjectInfoDisclosureService.deleteByKrId(krInfo.getId());
        okrObjectInfoRemindService.deleteByKrId(krInfo.getId());
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        insertKrPublicIds(krInfo,publicIds);
        insertKrRemindIds(sysUser.getName(),krInfo,remindIds);

        //日志留存
        okrLogService.insertOkrLog(2,2,krInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(krInfo),userId);
    }
    //获取本季度同步给我OKR的同事的列表

    @Override
    public Map<String, Object> getSyncUserListByMemberId(JSONObject data){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String openId = sysUser.getOpenId();
        Integer pageSize = data.getInteger("pageSize");
        if(null == pageSize||pageSize == 0){
            pageSize = 16;
        }
        Integer pageNumber = data.getInteger("pageNumber");
        if(null == pageNumber||pageNumber == 0){
            pageNumber = 1;
        }
        Integer year = data.getInteger("year");
        Integer quarter = data.getInteger("quarter");
        String name = data.getString("name");
        Map<String, Object> map = new HashMap<>();

        Integer todayQuarter = DateUtils.getQuarter(new Date());
        Integer todayYear = DateUtils.getYear(new Date());
        Integer isCurrentQuarter = 0;
        if(year ==null){
            year = todayYear;
        }
        if(null == quarter||quarter == 0){
            quarter = todayQuarter;
        }

        //如果不是当年当季度，返回O和KR的信息；是当季度，直接返回OKR四象限信息
        if(todayYear.equals(year) && todayQuarter.equals(quarter)){
            isCurrentQuarter = 1;
        }else{
            isCurrentQuarter = 0;
        }

        List<Map<String, Object>> mapList = new ArrayList<>();
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoService.getAllQuarterObject(name,year, quarter,(pageNumber-1)*pageSize,pageSize);
        for(OkrObjectInfo object:OkrObjectInfoList){
            Map<String, Object> okrmap = getOkrMap(object,openId,year,quarter);
            mapList.add(okrmap);
        }

        map.put("isCurrentQuarter",isCurrentQuarter);
        map.put("list",mapList);
        map.put("year",year);
        map.put("quarter",quarter);
        return map;
    }

    List<OkrKrInfo>checkKrList(List<OkrKrInfo> krList,String openId){
        List<OkrKrInfo>setList = new ArrayList<>();
        if(null!=krList&&!krList.isEmpty()){
            setList = new ArrayList<>();
            for(OkrKrInfo krInfo:krList){
                //1:查询当前kr使用可以被展示
                OkrKrInfo getKrInfo = krInfoservice.checkShow(krInfo,openId);
                if(null!=getKrInfo){
                    setList.add(getKrInfo);
                }
            }
        }
        return setList;
    }

    @Override
    public Map<String, Object> getMyFollowList(JSONObject data){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String openId = sysUser.getOpenId();
        Integer pageSize = data.getInteger("pageSize");
        if(null == pageSize||pageSize == 0){
            pageSize = 16;
        }
        Integer pageNumber = data.getInteger("pageNumber");
        if(null == pageNumber||pageNumber == 0){
            pageNumber = 1;
        }
        Integer year = data.getInteger("year");
        Integer quarter = data.getInteger("quarter");
        String name = data.getString("name");
        Map<String, Object> map = new HashMap<>();

        Integer todayQuarter = DateUtils.getQuarter(new Date());
        Integer todayYear = DateUtils.getYear(new Date());
        Integer isCurrentQuarter = 0;
        if(year ==null){
            year = todayYear;
        }
        if(null == quarter||quarter == 0){
            quarter = todayQuarter;
        }

        //如果不是当年当季度，返回O和KR的信息；是当季度，直接返回OKR四象限信息
        if(todayYear.equals(year) && todayQuarter.equals(quarter)){
            isCurrentQuarter = 1;
        }else{
            isCurrentQuarter = 0;
        }

        List<Map<String, Object>> mapList = new ArrayList<>();
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoService.getFollowQuarterObject(openId,name,year, quarter,(pageNumber-1)*pageSize,pageSize);
        for(OkrObjectInfo object:OkrObjectInfoList){
            Map<String, Object> okrmap = getOkrMap(object,openId,year,quarter);
            mapList.add(okrmap);
        }

        map.put("isCurrentQuarter",isCurrentQuarter);
        map.put("list",mapList);
        map.put("year",year);
        map.put("quarter",quarter);
        return map;
    }

    private Map<String, Object> getOkrMap(OkrObjectInfo object, String openId, Integer year, Integer quarter) {
        Map<String,Object>okrmap = new HashMap<>();
        List<OkrKrInfo> krList = okrKrInfoMapper.getKrByObjectId(object.getId(), object.getUserId(), year, quarter);
        List<OkrKrInfo>setList =checkKrList(krList,openId);
        QYWxUserInfo userInfo = wxUserInfoService.getByOpenId(object.getUserId());
        OkrFocusInfo focusInfo = focusInfoService.getByParams(object.getUserId(),openId);
        if(null == focusInfo){
            okrmap.put("follow",false);
        }else{
            okrmap.put("follow",true);
        }
        okrmap.put("krList", setList);
        okrmap.put("object", object);
        okrmap.put("name",userInfo.getName());
        return okrmap;
    }
    @Autowired
    private IOkrFocusInfoService focusInfoService;


    @Override
    public Map<String, Object> getMyDepartmentList(JSONObject data) {
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String openId = sysUser.getOpenId();
        Integer pageSize = data.getInteger("pageSize");
        if(null == pageSize||pageSize == 0){
            pageSize = 16;
        }
        Integer pageNumber = data.getInteger("pageNumber");
        if(null == pageNumber||pageNumber == 0){
            pageNumber = 1;
        }
        Integer year = data.getInteger("year");
        Integer quarter = data.getInteger("quarter");
        String name = data.getString("name");
        Map<String, Object> map = new HashMap<>();

        Integer todayQuarter = DateUtils.getQuarter(new Date());
        Integer todayYear = DateUtils.getYear(new Date());
        Integer isCurrentQuarter = 0;
        if(year ==null){
            year = todayYear;
        }
        if(null == quarter||quarter == 0){
            quarter = todayQuarter;
        }

        //如果不是当年当季度，返回O和KR的信息；是当季度，直接返回OKR四象限信息
        if(todayYear.equals(year) && todayQuarter.equals(quarter)){
            isCurrentQuarter = 1;
        }else{
            isCurrentQuarter = 0;
        }

        List<Map<String, Object>> mapList = new ArrayList<>();
        //1：查询自己所属部门下的用户信息
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoService.getDepartmentQuarterObject(openId,name,year, quarter,(pageNumber-1)*pageSize,pageSize);
        for(OkrObjectInfo object:OkrObjectInfoList){
            Map<String, Object> okrmap = getOkrMap(object,openId,year,quarter);
            mapList.add(okrmap);
        }

        map.put("isCurrentQuarter",isCurrentQuarter);
        map.put("list",mapList);
        map.put("year",year);
        map.put("quarter",quarter);
        return map;
    }
    @Autowired
    private IOkrWeekDataService weekDataService;
    @Override
    public Map<String, Object> getOkrDetail(JSONObject data) {
        Map<String, Object> result = new HashMap<>();
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String currentUserId = sysUser.getOpenId();
        Integer year = data.getInteger("year");
        Integer quarter = data.getInteger("quarter");
        Integer currentYear = DateUtils.getYear(new Date());
        if(null == year||year == 0){
            year = currentYear;
        }
        Integer currentQuarter = DateUtils.getQuarter(new Date());
        if(null == quarter||quarter == 0){
            quarter = currentQuarter;
        }
        String now = DateUtils.formatDate();
        //当前日期数据
        OkrWeekData weekData = weekDataService.getWeekByDate(now);
        List<Integer> months = DateUtils.getMonthsByQuarter(quarter);
        //根据月份数据获取周
        Map<Integer, Object> weeks = weekDataService.getWeeksByMonths(year,months);
        Map<String,Object> timeMap = new HashMap<>();
        timeMap.put("year", year);
        timeMap.put("quarter",quarter);
        timeMap.put("currentYear",weekData.getYear());
        timeMap.put("currentQuarter",weekData.getQuarter());
        timeMap.put("currentMonth",weekData.getMonth());
        timeMap.put("currentWeek",weekData.getWeek());
        timeMap.put("currentDate",weekData.getStartDate()+"至"+weekData.getEndDate());
        timeMap.put("months",months);
        timeMap.put("weeks",weeks);
        result.put("time", timeMap);

        OkrObjectInfo objectInfo =objectInfoService.getByParams(currentUserId,year,quarter);
        if(null == objectInfo){
            return result;
        }

        //周任务信息
        List<OkrTaskInfo> taskInfoList = okrTaskInfoService.getTaskInfo(objectInfo.getUserId(), year, quarter, 1);
        List<OkrTaskInfo> setTaskList =checkTaskInfo(taskInfoList,currentUserId);

        //月度任务信息
        List<OkrTaskInfo> taskFourWeekList = okrTaskInfoService.getTaskInfo(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter(), 2);
        List<OkrTaskInfo> settaskFourWeekList =checkTaskInfo(taskFourWeekList,currentUserId);

        //OKR信息
        List<OkrObjectInfo> OkrObjectInfoList = okrObjectInfoService.getQuarterObject(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());
        List<Map<String, Object>> okrList = new ArrayList<>();
        for (OkrObjectInfo okrObjectInfo:OkrObjectInfoList){
            Map<String, Object> okrMap = new HashMap<>();
            List<OkrKrInfo> krList = okrKrInfoMapper.getKrByObjectId(okrObjectInfo.getId(),objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());
            List<OkrKrInfo>setKrList =checkKrList(krList,currentUserId);
            okrMap.put("objectInfo",okrObjectInfo);
            okrMap.put("krList",setKrList);
            okrList.add(okrMap);
        }

        //状态信息
        List<OkrContentStatus> contentStatusList = okrContentStatusMapper.getContentStatusList(objectInfo.getUserId(), objectInfo.getYear(), objectInfo.getQuarter());

        result.put("taskInfoList",transformTaskInfo(setTaskList, 1));
        result.put("taskFourWeekList",transformTaskInfo(settaskFourWeekList, 2));
        result.put("okr",okrList);
        result.put("status", contentStatusList);
        return result;
    }


    public Map<Integer, List<OkrTaskInfo>> transformTaskInfo(List<OkrTaskInfo> taskInfoList, Integer type){

        Map<Integer, List<OkrTaskInfo>> resultMap = new HashMap<>();
        // 1周  2月
        if(type == 1){
            taskInfoList.forEach(taskInfo -> {
                Integer key = Check.isNull(taskInfo.getWeek()) ? 0 : taskInfo.getWeek();
                if(resultMap.containsKey(key)){
                    resultMap.get(key).add(taskInfo);
                }else{
                    List<OkrTaskInfo> list = new ArrayList<>();
                    list.add(taskInfo);
                    resultMap.put(key, list);
                }
            });
        }

        if(type == 2){
            taskInfoList.forEach(taskInfo -> {
                Integer key = Check.isNull(taskInfo.getMonth()) ? 0 : taskInfo.getMonth();
                if(resultMap.containsKey(key)){
                    resultMap.get(key).add(taskInfo);
                }else{
                    List<OkrTaskInfo> list = new ArrayList<>();
                    list.add(taskInfo);
                    resultMap.put(key, list);
                }
            });
        }
        //联调测试有问题的话做转换
        //String json = JSONObject.toJSONString(resultMap);

        return resultMap;
    }

    @Override
    public void deleteObjectInfoById(OkrObjectInfo objectInfo){
        okrObjectInfoService.deleteObjectInfoById(objectInfo);
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        okrLogService.insertOkrLog(3,1,objectInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(objectInfo),userId);
    }

    @Override
    public void deleteKrById(OkrKrInfo krInfo){
        okrKrInfoMapper.deleteKrById(krInfo);
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        okrLogService.insertOkrLog(3,2,krInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(krInfo),userId);
    }

    @Override
    public void deleteContentStatusById(OkrContentStatus content){
        okrContentStatusMapper.deleteContentStatuaById(content);
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        okrLogService.insertOkrLog(3,5,content.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(content),userId);
    }
    @Override
    public void deleteTaskInfoById(OkrTaskInfo okrTaskInfo){
        okrTaskInfoService.deleteTaskInfoById(okrTaskInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        Integer opetation = 1;
        //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        if(okrTaskInfoService.selectById(okrTaskInfo.getId()).getType()==1){
            opetation = 3;
        }else{
            opetation = 4;
        }
        okrLogService.insertOkrLog(3,opetation,okrTaskInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(okrTaskInfo),userId);
    }
    @Override
    public void keepTaskInfoById(OkrTaskInfo okrTaskInfo){
        okrTaskInfoService.keepTaskInfoById(okrTaskInfo);

        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        //日志留存
        Integer opetation = 1;
        //日志类型：1.目标2.关键成果3.本周任务4.未来四周任务5.状态指标
        if(okrTaskInfoService.selectById(okrTaskInfo.getId()).getType()==1){
            opetation = 3;
        }else{
            opetation = 4;
        }
        okrLogService.insertOkrLog(4,opetation,okrTaskInfo.getId(),DateUtils.getYear(new Date()),DateUtils.getQuarter(new Date()),JSONObject.toJSONString(okrTaskInfo),userId);
    }
    @Autowired
    private IOkrObjectInfoDisclosureService okrObjectInfoDisclosureService;
    @Autowired
    private IOkrObjectInfoRemindService okrObjectInfoRemindService;
    @Override
    public void insertOkrKrInfo(OkrKrInfo krInfo){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String userId = sysUser.getOpenId();
        Integer quarter = DateUtils.getQuarter(new Date());
        Integer year = DateUtils.getYear(new Date());
        krInfo.setYear(year);
        krInfo.setQuarter(quarter);
        krInfo.setUserId(userId);
        okrKrInfoMapper.insertOkrKrInfo(krInfo);

        insertKrPublicIds(krInfo,krInfo.getPublicIds());
        insertKrRemindIds(sysUser.getName(),krInfo,krInfo.getRemindIds());

        //日志留存
        okrLogService.insertOkrLog(1,2,krInfo.getId(),year,quarter,JSONObject.toJSONString(krInfo),userId);

    }

    @Override
    public List<Integer> getObjectYear(){
        return okrObjectInfoService.getObjectYear();
    }



}
