package cn.com.ctop.okr.service.impl;

import cn.com.ctop.okr.dto.UserInfoDto;
import cn.com.ctop.okr.entity.CorpUserInfo;
import cn.com.ctop.okr.entity.QYWxDepartInfo;
import cn.com.ctop.okr.entity.QYWxUserDepartMap;
import cn.com.ctop.okr.entity.QYWxUserInfo;
import cn.com.ctop.okr.mapper.CorpUserInfoMapper;
import cn.com.ctop.okr.mapper.QYWxDepartInfoMapper;
import cn.com.ctop.okr.mapper.QYWxUserInfoMapper;
import cn.com.ctop.okr.qywx.provider.AuthProvider;
import cn.com.ctop.okr.qywx.provider.DepartUserProvider;
import cn.com.ctop.okr.service.QYWxService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
@Slf4j
public class QYWxServiceImpl extends ServiceImpl<CorpUserInfoMapper, CorpUserInfo> implements QYWxService {

    @Autowired
    private QYWxDepartInfoMapper wxDepartInfoMapper;
    @Autowired
    private QYWxUserInfoMapper wxUserInfoMapper;

    @Override
    public void getWxDepartAndUserInfo() throws Exception {
        String token = queryWxToken();  //获取token
        //String token = "dtGm-D6u-nO3wGM8GAlh6OWI1BzzZQBZQPHnA24S8NHpMrECPp8KNjY4EAZcKaJoSTZudQsVO3Ox7yyeJmO9ZOa02n4N4FVxIX-CQNLSNsgoafC75jeecDPPBhhQX7TesqFWoumcWVso2yNycQ_Kal1S1jvS0SBY1ggwsWe6OZhH_k0XElPEop6D50Ai6NFX1eOoTFMvUsSZecIiZRRFiQ";
        System.out.println(token);
        queryUserDeparts(token);  //更新部门列表
        queryUserInfo(token);  //更新员工信息
        updateUserOpenId(token);  //转换员工userid到openId
    }

    /**
     * 获取token
     * @return
     * @throws Exception
     */
    public String queryWxToken() throws Exception {
        return AuthProvider.getQYWXAccessToken();
    }

    /**
     * 获取企业微信公司所有部门
     * @return
     */
    public void queryUserDeparts(String token) throws Exception{
        JSONObject resultJson = DepartUserProvider.getCorpDepartList(null,token);
        Integer errcode = resultJson.getInteger("errcode");
        if(errcode == 0){
            JSONArray departmentArray = resultJson.getJSONArray("department");

            if(departmentArray == null){
                throw new Exception("获取部门失败，返回值：" + JSON.toJSONString(resultJson));
            }
            int size = departmentArray.size();
            for(int i=0; i < size; i++){
                JSONObject json = departmentArray.getJSONObject(i);
                QYWxDepartInfo departInfo = new QYWxDepartInfo();
                departInfo.setId(json.getLong("id"));
                departInfo.setDepartName(json.getString("name"));
                departInfo.setDepartOrder(json.getLong("order"));
                departInfo.setParentid(json.getLong("parentid"));
                wxDepartInfoMapper.insertWxDepartInfo(departInfo);
            }
        }
    }

    /**
     * 获取企业微信员工信息
     * @param token
     * @throws Exception
     */
    public void queryUserInfo(String token) throws Exception {
        JSONObject resultJson = DepartUserProvider.getUserDetailInfoByDepartId(1,token);
        Integer errcode = resultJson.getInteger("errcode");
        if(errcode == 0){
            JSONArray userArray = resultJson.getJSONArray("userlist");
            if(userArray == null){
                throw new Exception("获取员工明细失败，返回值：" + JSON.toJSONString(resultJson));
            }

            Integer size = userArray.size();
            for(int j=0; j<size; j++){
                JSONObject json = userArray.getJSONObject(j);
                QYWxUserInfo userInfo = new QYWxUserInfo();
                String userid = json.getString("userid");
                userInfo.setMobile(json.getString("mobile"));
                userInfo.setAvatar(json.getString("avatar"));
                userInfo.setMainDepartment(json.getLong("main_department"));
                userInfo.setUserid(userid);
                userInfo.setThumbAvatar(json.getString("thumb_avatar"));
                userInfo.setName(json.getString("name"));
                userInfo.setQrCode(json.getString("qr_code"));
                userInfo.setEmail(json.getString("email"));
                userInfo.setStatus(json.getInteger("status"));

                wxUserInfoMapper.replaceIntoWxUserInfo(userInfo);

                JSONArray departArray = json.getJSONArray("department");
                Integer departSize = departArray.size();

                wxUserInfoMapper.deleteUserDepartMapByUserid(userid);
                for(int k=0; k<departSize ; k++){
                    QYWxUserDepartMap map = new QYWxUserDepartMap();
                    Long departId = departArray.getLong(k);
                    map.setDepartId(departId);
                    map.setUserid(userid);
                    wxUserInfoMapper.insertUserDeopartMap(map);
                }
            }
        }
    }

    /**
     *  userid转换为openId
     * @param token
     * @throws Exception
     */
    public void updateUserOpenId(String token) {
        List<String> userList = wxUserInfoMapper.getUseridList();
        for(String userid:userList){
            try{
                JSONObject resultJson = DepartUserProvider.convertUserIdToOpenId(userid, token);

                Integer errcode = resultJson.getInteger("errcode");
                if(errcode == 0){
                    String openId = resultJson.getString("openid");
                    wxUserInfoMapper.updateUserInfoByUserid(userid, openId);
                    wxUserInfoMapper.updateUserDepartMapOpenIdByUserid(userid, openId);
                }else{
                    //throw new Exception("userid转换为openId出错啦，resultJson为" + resultJson);
                    System.out.println(userid + " : userid转换为openId出错啦" + resultJson);
                    //log.info("1112");
                }
            }catch(Exception e){

            }
        }

    }

    //根据名字模糊查询企业微信用户
    @Override
    public List<UserInfoDto> getUserListByName(String name){
        QYWxUserInfo sysUser = (QYWxUserInfo) SecurityUtils.getSubject().getPrincipal();
        String openId = sysUser.getOpenId();
        return wxUserInfoMapper.getUserListByName(name, openId);
    }

    //根据名字模糊查询企业微信用户
    @Override
    public QYWxUserInfo getUserInfoByUserid(String userid){
        return wxUserInfoMapper.getUserInfoByUserid(userid);
    }


}
